procedure mkobs (fitsfile, errfile, specfile)

# If the FITS file is a standard IRAF multispec file (use format=lin or
# or format=log depending on the pixel wavelength units), or a standard SDSS 
# spectro1d format=c1d (Chicago1D) or specBS format=p1d (Princeton1D) file,
# then using errfile="" will extract the error array correctly.  If you specify
# the errfile instead, you must specify it EXACTLY, e.g. "x.fits[0][*,3]".
# 
# If you have two one-dimensional FITS files, one with the spectrum and one
# with the error array, use the default format=c1d, it will work correctly.
#
# If you have no error array, just use the name of the spectrum again.
#
# **add option to create new fits file of just spectrum and error array
# e.g. imcopy 0442/spPlate-0442-51882.fits[0][*,528] spPlate-0442-51882_528.fits

string fitsfile     {prompt="Root name of FITS spectrum (no extension)"}
string errfile=""   {prompt="Root name of FITS error array"}
string specfile=""  {prompt="Observed spectrum text file to create"}
string format="c1d" {enum="c1d|p1d|lin|log",prompt="Format (c1d|p1d|lin|log)"}
bool normalize=no   {prompt="Normalize spectra by 1E-17?"}
int smooth=1        {prompt="Boxcar smooth by how many pixels?"}
struct *list

begin

string phistdir       # pathname of PHIST source code directory
string temp1, fitsnombre, nombre, otronombre, nuevonombre, malnombre, fmt
string extfits, prepend, crval1, crnew1, newname, temp0, mal_nombre, temp2
real zed, xmin, xmax, ymin, ymax, junk, obsmin, obsmax, conclusion, numero
real denominator, numerator, colorexcess, z
bool miny, errores, falsificacion, princeton, eco, go, libel, flatten, excurve
int alisar

fitsnombre = fitsfile
malnombre = errfile
nuevonombre = specfile
fmt = format
flatten = normalize
alisar=smooth

temp0 = mktemp("temp0")
temp1 = mktemp("temp1")
temp2 = mktemp("temp2")
nombre=temp1
numerator=1.0E17

#--crap needed for smoothing
mal_nombre=malnombre
if (malnombre=="") {
	if (fmt=="p1d") {
		mal_nombre=fitsnombre//"[1]"
	} else if (fmt=="c1d") {
		mal_nombre=fitsnombre//"[0][*,3]"
	} else if ((fmt=="lin") || (fmt=="log")) {
		mal_nombre=fitsnombre//"[*,1,4]"
	}
}

#--smoothing
if (alisar<=0) { alisar=1 }
if ( (alisar!=1) && (!imaccess(fitsnombre//"sm"//alisar)) ) {
        temp0=mktemp("temp0")
        temp2=mktemp("temp2")
        print("s\ni\nq\n",>>temp0)
	splot(fitsnombre//"[0]",line=1,cursor=temp0,new_im=fitsnombre//"sm"//alisar,boxsize=alisar)
        splot(mal_nombre,line=1,cursor=temp0,new_im=temp2,boxsize=alisar)
	if(fmt=="p1d"){
     	   imar(temp2,"*",alisar,fitsnombre//"errsm"//alisar,divz=0,ver-,noact-)
	} else {
     imar(temp2,"/",sqrt(alisar),fitsnombre//"errsm"//alisar,divz=0,ver-,noact-)
	}
        del (temp0, ver-) ; imdel (temp2, ver-)
	if(malnombre!=""){
		malnombre=malnombre//"sm"//alisar
		imren(fitsnombre//"errsm"//alisar,malnombre,ver-)
		fitsnombre=fitsnombre//"sm"//alisar
 print("Created "//fitsnombre//" & "//malnombre)
	} else {
		if (fmt=="p1d") {
 print("Created "//fitsnombre//"sm"//alisar//" & "//fitsnombre//"errsm"//alisar)
        	} else if (fmt=="c1d") {
                	imcopy(fitsnombre//"errsm"//alisar//"[0][*,3]",fitsnombre//"sm"//alisar//"[0][*,3]")
                	imdel(fitsnombre//"errsm"//alisar,ver-)
 			print("Created "//fitsnombre//"sm"//alisar)
        	} else if ((fmt=="lin") || (fmt=="log")) {
 print("Created "//fitsnombre//"sm"//alisar//" & "//fitsnombre//"errsm"//alisar)
        	}
		fitsnombre=fitsnombre//"sm"//alisar
        }
}

if (nuevonombre=="") { nuevonombre=fitsnombre//".obs" }

#--idlspec2d format
if (fmt=="p1d") {
#--create spectrum text file, if needed
	junk=0
	if (!access(nuevonombre)) {
# --list out flux & 1/sigma^2 in log10(lambda)
		if (!access(nombre)) { 
			listpix (fitsnombre//"[0]", wcs="world", >> nombre)
			if (malnombre=="") {
				listpix (fitsnombre//"[1]", wcs="world", >> nombre//"_errs") 
			} else {
				listpix (malnombre, wcs="world", >> nombre//"_errs") 
			}
		}
# --convert to linear lambda in observed frame, and to sigma from 1/sigma^2
		filecalc (nombre//","//nombre//"_errs", "10.**$1@1;$2@1;(1./sqrt($2@2))", format="%9.3f %10.6f %10.6f", >> nuevonombre)
		del (nombre//"_errs", ver-, >& "dev$null")
		print("Created "//nuevonombre)
	} else {
		print (nuevonombre//" already exists!")
	}
#--spectro1d format
} else if (fmt=="c1d") {
#--create spectrum text file, if needed
        junk=0
        if (!access(nuevonombre)) {
# --list out flux & sigma in log10(lambda)
                if (!access(nombre)) {
                	listpix (fitsnombre//"[0][*,1]", wcs="world", >> nombre)
			if (malnombre=="") {
                		listpix (fitsnombre//"[0][*,3]", wcs="world", >> nombre//"_errs")
                        } else {
                		listpix (malnombre, wcs="world", >> nombre//"_errs")
                        }
                }
# --convert to linear lambda in observed frame
		if (flatten) {
                	filecalc (nombre//","//nombre//"_errs", "10.**$1@1;$2@1*"//numerator//";$2@2*"//numerator, format="%9.3f %10.6f %10.6f", >> nuevonombre)
		} else {
                	filecalc (nombre//","//nombre//"_errs", "10.**$1@1;$2@1;$2@2", format="%9.3f %10.6f %10.6f", >> nuevonombre)
		}
                del (nombre//"_errs", ver-, >& "dev$null")
		print("Created "//nuevonombre)
	} else {
		print (nuevonombre//" already exists!")
        }
#--IRAF format
} else if (fmt=="lin" || fmt=="log") {
#--create spectrum text file, if needed
        junk=0
        if (!access(nuevonombre)) {
# --list out flux & sigma in log10(lambda)
                if (!access(nombre)) {
                        listpix (fitsnombre//"[*,1,1]", wcs="world", >> nombre)
                        if (malnombre=="") {
                                listpix (fitsnombre//"[*,1,4]", wcs="world", >> nombre//"_errs")
                        } else {
				listpix (malnombre, wcs="world", >> nombre//"_errs")
                        }
                }
# --convert to linear lambda in observed frame, if needed
		if (fmt=="log") {
			if (flatten) {
				filecalc (nombre//","//nombre//"_errs", "10.**$1@1;$2@1*"//numerator//";$2@2*"//numerator, format="%9.3f %10.6f %10.6f", >> nuevonombre)
			} else {
				filecalc (nombre//","//nombre//"_errs", "10.**$1@1;$2@1;$2@2", format="%9.3f %10.6f %10.6f", >> nuevonombre)
			}
                	del (nombre//"_errs", ver-, >& "dev$null")
        	} else {
			if (flatten) {
				filecalc (nombre//","//nombre//"_errs", "$1@1;$2@1*"//numerator//";$2@2*"//numerator, format="%9.3f %10.6f %10.6f", >> nuevonombre)
			} else {
				filecalc (nombre//","//nombre//"_errs", "$1@1;$2@1;$2@2", format="%9.3f %10.6f %10.6f", >> nuevonombre)
			}
                        del (nombre//"_errs", ver-, >& "dev$null")
		}
		print("Created "//nuevonombre)
        } else {
                print (nuevonombre//" already exists!")
        }
}

del (temp1, ver-, >& "dev$null")
print ("3800 0", >> temp1)
concat (temp1//","//nuevonombre) | graph (pointmode-)
#graph (nuevonombre, pointmode-)
fields (nuevonombre, "1,3") | graph (pointmode-, append+)
del (temp1, ver-, >& "dev$null")

end

